@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.38 2004/01/12 15:58:34 bazsi Exp $ */
+ * $Id: io.c,v 1.44 2005/05/03 15:49:35 bazsi Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -52,6 +52,10 @@
 #include <arpa/inet.h>
 #include <signal.h>
 
+#if HAVE_ALLOCA_H
+# include  <alloca.h>
+#endif
+
 static void do_mark_callouts(struct ol_queue *q, void (*mark)(struct ol_object *o));
 static void do_free_callouts(struct ol_queue *q);
 
@@ -136,6 +140,11 @@ int io_iter(struct io_backend *b)
 					close(fd->fd);
 				}
 				/* Unlink this fd */
+				
+				/* skip newly added fds */
+				while (*fd_p && *fd_p != fd)
+					fd_p = &((*fd_p)->next);
+				assert(fd_p);
 				*fd_p = fd->next;
 				continue;
 			}
@@ -203,8 +212,12 @@ int io_iter(struct io_backend *b)
         }
         else {
                 gc_maybe(&b->super, 1);
-                if (nfds == 0)
-	        	res = poll(NULL, 0, timeout < 0 ? 60000 : timeout * 1000);
+                if (nfds == 0) {
+                	struct pollfd dummy; 
+                	
+                	/* this hack is needed on MacOSX where the poll emulation does not handle the fdarray being NULL */
+	        	res = poll(&dummy, 0, timeout < 0 ? 60000 : timeout * 1000);
+	        }
         }
 
 	if (res < 0)
@@ -420,7 +433,9 @@ static void read_callback(struct nonblocking_fd *fd)
 	int res;
 
 	struct fd_read r =
-	{ { STACK_HEADER, do_read, do_recv }, fd->fd };
+	{ { STACK_HEADER, do_read, do_recv }, 0 };
+	
+	r.fd = fd->fd;
 
 	/* The handler function may install a new handler */
 	res = READ_HANDLER(self->handler,
@@ -508,7 +523,10 @@ static void write_callback(struct nonblocking_fd *fd)
 	int res;
 
 	struct fd_write w =
-	{ { STACK_HEADER, do_write }, fd->fd, self->fsync };
+	{ { STACK_HEADER, do_write }, 0, 0 };
+	
+	w.fd = fd->fd;
+	w.fsync = self->fsync;
 
 	assert(self->buffer);
 
@@ -680,7 +698,9 @@ void init_file(struct io_backend *b, struct nonblocking_fd *f, int fd,
 int blocking_read(int fd, struct read_handler *handler)
 {
 	struct fd_read r =
-	{ { STACK_HEADER, do_read }, fd };
+	{ { STACK_HEADER, do_read }, 0 };
+	
+	r.fd = fd;
 
 	for (;;)
 	{
@@ -735,7 +755,7 @@ get_inaddr(struct sockaddr_in	* addr,
 			 * inet_aton rather than inet_addr; see the GNU libc
 			 * documentation. */
 # ifndef INADDR_NONE
-# define INADDR_NONE ((unsigned long int) -1)
+# define INADDR_NONE ((in_addr_t) -1)
 # endif /* !INADDR_NONE */
 			addr->sin_addr.s_addr = inet_addr(host);
 		if (addr->sin_addr.s_addr == INADDR_NONE)
@@ -852,7 +872,7 @@ static int inet_address2sockaddr_in(struct address_info *c, int sasize, struct s
 	}
 	if (!tcp_addr(in,
 		     a && a->ip ? a->ip->length : 7, 
-		     a && a->ip ? (char *) a->ip->data : "0.0.0.0",
+		     a && a->ip ? (UINT8 *) a->ip->data : (UINT8 *) "0.0.0.0",
 		     a ? a->port : 0))
 		return 0;
 	return 1;
@@ -929,7 +949,7 @@ static int unix_address2sockaddr_un(struct address_info *c, int sasize, struct s
 	struct sockaddr_un *un = (struct sockaddr_un *) sa;
 	if (sasize != sizeof(*un))
 		return 0;
-	strncpy(un->sun_path, a->path->data, MIN(a->path->length + 1, sizeof(un->sun_path)));
+	strncpy(un->sun_path, (char *) a->path->data, LIBOL_MIN(a->path->length + 1, sizeof(un->sun_path)));
 	un->sun_family = AF_UNIX;
 	return 1;
 }
@@ -1355,7 +1375,7 @@ int reopen_fd(struct nonblocking_fd *fd)
 	if ((oldmode & O_WRONLY) || (oldmode & O_RDWR))
 		oldmode |= O_CREAT | O_APPEND;
 
-	if ((newfd = open(fd->fname->data, oldmode)) == -1)
+	if ((newfd = open((char *) fd->fname->data, oldmode)) == -1)
 	{
 		werror("reopen_fd: open failedfor %S: %z\n",
 			fd->fname, strerror(errno));
